import { Users } from '../db/mongodb';
import type User from '../../types/User';
import type Resources from '../../types/Resources';
import type Building from '../../types/Building';

export const getAllUsers = async () => {
    const users = await Users();
    return users.find({}).toArray() as Promise<User[]>;
}

export const createUser = async (user: User) => {
    const newUser = await (await Users()).insertOne(user);
    return newUser;
}

export const getUserByNickOrEmail = async (searchString: string) => {
    const users = await Users();
    return users.findOne({
        $or: [
            { username: searchString },
            { email: searchString }
        ]
    }) as Promise<User | null>;
}

export const getUserResources = async (username: string): Promise<Resources> => {
    const users = await Users();
    const user = await users.findOne({ username });

    const defaultResources: Resources = {
        coal: 0,
        iron: 0,
        gold: 0
    };

    return user?.resources === undefined ? defaultResources : user?.resources;
}

export const updateUserResources = async (username: string, resources: any) => {
    const users = await Users();
    await users.updateOne({ username }, {
        $set: {
            resources
        }
    });
}

export const getUserBuildings = async (username: string): Promise<Array<Building>> => {
    const users = await Users();
    const user = await users.findOne({ username });

    const buildings: Array<Building> = [];

    if (user?.buildings !== undefined) {
        user?.buildings.forEach((building: Building) => {
            buildings.push(building);
        });
    }

    return buildings;
}

export const createOrUpgradeBuilding = async (username: string, building: Building) => {
    const users = await Users();

    const result = await users.updateOne(
        { username, "buildings.name": building.id },
        { $set: { "buildings.$.level": building.level } }
    );

    if (result.modifiedCount === 0) {
        await users.updateOne({ username },
            { $push: { buildings: building } }
        );
    }
}