import type AccessToken from "../../types/AccessToken";
import { getAccessToken } from "../db/accessTokens";

export default async function validateAccessToken(request: Request): Promise<Response | AccessToken> {
    let accessToken = request.url.split("?")[1]?.split("&").filter((x) => x.split("=")[0] === "token")[0].split("=")[1];

    const cookies = request.headers.get("Cookie")?.split(";").map((x) => x.trim().split("=")) ?? [];

    if(accessToken === undefined) accessToken = cookies.filter((x) => x[0] === "sessionToken")[0]?.[1];

    if(accessToken === undefined) return new Response(
        JSON.stringify({
            code: 401,
            message: "Unauthorized",
            error: "Missing Access Token"
        }), { status: 401 }
    );

    const response = await getAccessToken(accessToken);

    if(response === null) return new Response(
        JSON.stringify({
            code: 401,
            message: "Unauthorized",
            error: "Invalid Access Token"
        }), { status: 401 }
    );
    
    if(response.createdAt.getTime() > Date.now()) return new Response(
        JSON.stringify({
            code: 403,
            message: "Forbidden",
            data: "Access token is invalid for user " + response.username + ", are you travelling in time?"
        }), { status: 403 }
    );
    
    if(response.expiresAt !== null && response.expiresAt.getTime() < Date.now()) return new Response(
        JSON.stringify({
            code: 403,
            message: "Forbidden",
            data: "Access token is invalid for user " + response.username + ", token expired"
        }), { status: 403 }
    );

    return response;
}